package lcsb.mapviewer.converter.model.celldesigner;

import lcsb.mapviewer.commands.CopyCommand;
import lcsb.mapviewer.commands.CreateHierarchyCommand;
import lcsb.mapviewer.model.Project;
import lcsb.mapviewer.model.map.kinetics.SbmlFunction;
import lcsb.mapviewer.model.map.kinetics.SbmlParameter;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelComparator;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.species.Element;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertNotEquals;
import static org.junit.Assert.assertTrue;

public class CopyCommandForCellDesignerFilesTest extends CellDesignerTestFunctions {

  private final ModelComparator comparator = new ModelComparator();

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  private final int counter = 0;

  @Test
  public void testCopyModel() throws Exception {
    Model model = getModelForFile("testFiles/sample.xml");
    Model copy = new CopyCommand(model).execute();

    assertEquals(0, comparator.compare(model, copy));
  }

  @Test
  public void testCopyModelWithKinetics() throws Exception {
    Model model = getModelForFile("testFiles/kinetics_with_compartment.xml");
    Model copy = new CopyCommand(model).execute();

    assertEquals(0, comparator.compare(model, copy));
    for (final Reaction reaction : copy.getReactions()) {
      if (reaction.getKinetics() != null) {
        for (final Element element : reaction.getKinetics().getElements()) {
          assertTrue("Element in the copy doesn't belong to copy", copy.getElements().contains(element));
        }
        for (final SbmlFunction function : reaction.getKinetics().getFunctions()) {
          assertTrue("Function in the copy doesn't belong to copy", copy.getFunctions().contains(function));
        }
        for (final SbmlParameter parameter : reaction.getKinetics().getParameters()) {
          if (parameter.getParameterId().equals("k2")) {
            assertTrue("Global parameter in the function copy doesn't belong to copy",
                copy.getParameters().contains(parameter));
          }
        }
      }
    }
  }

  @Test
  public void testCopyModel3() throws Exception {
    Model model = getModelForFile("testFiles/complex_with_state.xml");

    Model copy = new CopyCommand(model).execute();

    Model copy2 = serializeModel(copy);

    // check if after conversion to xml everything works
    assertEquals(0, comparator.compare(copy, copy2));
  }

  @Test
  public void testCopyModel4() throws Exception {
    Model model = getModelForFile("testFiles/problematic_description.xml");

    Model copy = new CopyCommand(model).execute();

    Model copy2 = serializeModel(copy);

    // check if after conversion to xml everything works
    assertEquals(0, comparator.compare(copy, copy2));
  }

  @Test
  public void testCopyModelWithArtificialAliases() throws Exception {
    Model model = getModelForFile("testFiles/artificial_compartments.xml");
    new CreateHierarchyCommand(model, 2, 2).execute();

    Model copy = new CopyCommand(model).execute();

    Model copy2 = serializeModel(copy);

    new CreateHierarchyCommand(copy2, 2, 2).execute();

    // check if after conversion to xml everything works
    assertEquals(0, comparator.compare(copy, copy2));
  }

  @Test
  public void testCopyModelWithProject() throws Exception {
    Model originalModel = getModelForFile("testFiles/sample.xml");
    Project project = new Project();
    project.addModel(originalModel);

    Model colorModel = new CopyCommand(originalModel).execute();

    assertNotEquals(originalModel.getProject(), colorModel.getProject());
  }

}
