package lcsb.mapviewer.commands;

import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.tests.TestUtils;
import lcsb.mapviewer.common.tests.UnitTestFailedWatcher;
import lcsb.mapviewer.model.cache.UploadedFileEntry;
import lcsb.mapviewer.model.graphics.HorizontalAlign;
import lcsb.mapviewer.model.graphics.PolylineData;
import lcsb.mapviewer.model.graphics.VerticalAlign;
import lcsb.mapviewer.model.map.layout.graphics.Glyph;
import lcsb.mapviewer.model.map.layout.graphics.LayerImage;
import lcsb.mapviewer.model.map.layout.graphics.LayerRect;
import lcsb.mapviewer.model.map.layout.graphics.LayerText;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelFullIndexed;
import lcsb.mapviewer.model.map.reaction.Product;
import lcsb.mapviewer.model.map.reaction.Reactant;
import lcsb.mapviewer.model.map.reaction.Reaction;
import lcsb.mapviewer.model.map.reaction.type.TransportReaction;
import lcsb.mapviewer.model.map.species.Complex;
import lcsb.mapviewer.model.map.species.Element;
import lcsb.mapviewer.model.map.species.Gene;
import lcsb.mapviewer.model.map.species.GenericProtein;
import lcsb.mapviewer.model.map.species.Protein;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.junit.Rule;

import java.awt.Color;
import java.awt.geom.Point2D;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

public abstract class CommandTestFunctions extends TestUtils {

  private static final Map<String, Model> models = new HashMap<>();
  protected static final double EPSILON = Configuration.EPSILON;

  @Rule
  public UnitTestFailedWatcher unitTestFailedWatcher = new UnitTestFailedWatcher();
  protected static Logger logger = LogManager.getLogger();

  private int counter;

  protected Model createSimpleModel() {
    final Model model = new ModelFullIndexed(null);
    model.setName(faker.name().fullName());

    final GenericProtein alias = new GenericProtein("alias_id");
    assignCoordinates(alias);
    alias.setZ(2);

    alias.setNotes(null);
    final List<String> list = new ArrayList<>();
    list.add("synonym");
    alias.addSynonyms(list);
    final List<String> list2 = new ArrayList<>();
    list2.add("f_symbol");
    alias.setFormerSymbols(list2);

    final Complex complexAlias = new Complex("complex_alias_id");
    assignCoordinates(complexAlias);
    complexAlias.setZ(3);
    model.addElement(complexAlias);

    complexAlias.addSpecies(alias);

    model.addElement(alias);

    return model;
  }

  protected Reaction createReaction(final Protein protein, final Protein protein2) {
    final Reaction reaction = new TransportReaction("" + counter++);
    reaction.addReactant(new Reactant(protein));
    reaction.addProduct(new Product(protein2));
    return reaction;
  }

  protected Reaction createReactionWithLayout(final Protein protein, final Protein protein2) {
    final Reaction reaction = new TransportReaction("" + counter++);
    Reactant reactant = new Reactant(protein);
    reactant.setLine(new PolylineData(new Point2D.Double(protein.getX(), protein.getY()), protein.getCenter()));
    Product product = new Product(protein2);
    product.setLine(new PolylineData(new Point2D.Double(protein2.getX(), protein2.getY()), protein2.getCenter()));
    reaction.addReactant(reactant);
    reaction.addProduct(product);
    reaction.setLine(new PolylineData(protein.getCenter(), protein2.getCenter()));
    return reaction;
  }

  protected GenericProtein createProtein() {
    final GenericProtein result = new GenericProtein("" + counter++);
    result.setWidth((Double) null);
    result.setHeight((Double) null);
    result.setX((Double) null);
    result.setY((Double) null);
    return result;
  }

  protected GenericProtein createProteinWithLayout() {
    final GenericProtein result = createProtein();
    result.setWidth(faker.number().numberBetween(100, 200));
    result.setHeight(faker.number().numberBetween(100, 200));
    result.setX(faker.number().numberBetween(100, 200));
    result.setY(faker.number().numberBetween(100, 200));
    return result;
  }

  protected Gene createGene() {
    final Gene result = new Gene("" + counter++);
    return result;
  }

  protected Complex createComplex() {
    final Complex result = new Complex("" + counter++);
    result.setWidth((Double) null);
    result.setHeight((Double) null);
    result.setX((Double) null);
    result.setY((Double) null);
    return result;
  }

  private int z = 1;

  protected void assignCoordinates(final Element element) {
    assignCoordinates(1, 1, 10, 20, element);
  }

  protected void assignCoordinates(final double x, final double y, final double width, final double height, final Element element) {
    element.setX(x);
    element.setY(y);
    element.setZ(z++);
    element.setWidth(width);
    element.setHeight(height);
    element.setNameX(x);
    element.setNameY(y);
    element.setNameWidth(width);
    element.setNameHeight(height);
    element.setNameVerticalAlign(VerticalAlign.MIDDLE);
    element.setNameHorizontalAlign(HorizontalAlign.CENTER);
  }

  protected LayerRect createLayerRect() {
    LayerRect layerRect = new LayerRect();
    layerRect.setBorderColor(Color.BLUE);
    layerRect.setFillColor(Color.YELLOW);
    return layerRect;
  }

  protected LayerImage createLayerImage() {
    LayerImage layerImage = new LayerImage();
    layerImage.setGlyph(createGlyph());
    return layerImage;
  }

  protected LayerText createLayerText() {
    LayerText layerText = new LayerText();
    layerText.setGlyph(createGlyph());
    layerText.setFillColor(Color.YELLOW);
    layerText.setNotes(faker.lorem().sentence());
    return layerText;
  }

  protected Glyph createGlyph() {
    Glyph glyph = new Glyph();
    glyph.setFile(createFile());
    return glyph;
  }

  private UploadedFileEntry createFile() {
    UploadedFileEntry file = new UploadedFileEntry();
    file.setOriginalFileName(faker.file().fileName());
    return file;
  }


}
