package lcsb.mapviewer.commands;

import lcsb.mapviewer.model.map.compartment.PathwayCompartment;
import lcsb.mapviewer.model.map.layout.graphics.Layer;
import lcsb.mapviewer.model.map.layout.graphics.LayerRect;
import lcsb.mapviewer.model.map.layout.graphics.LayerText;
import lcsb.mapviewer.model.map.model.Model;
import lcsb.mapviewer.model.map.model.ModelFullIndexed;
import lcsb.mapviewer.model.map.species.Protein;
import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;

public class CreateHierarchyCommandTest extends CommandTestFunctions {

  @Before
  public void setUp() throws Exception {
  }

  @After
  public void tearDown() throws Exception {
  }

  @Test
  public void testCreatePathwayFromRectangle() {
    Model model = new ModelFullIndexed(null);

    Layer layer = new Layer();
    layer.setName(CreateHierarchyCommand.PATHWAY_LAYER_NAME);
    LayerRect layerRect = createLayerRect();
    layer.addLayerRect(layerRect);
    model.addLayer(layer);

    CreateHierarchyCommand command = new CreateHierarchyCommand(model, 4, 80);
    command.createArtificials();

    PathwayCompartment pathway = (PathwayCompartment) model.getCompartments().get(0);

    assertEquals(pathway.getBorderColor(), pathway.getFontColor());
  }

  @Test
  public void testCreatePathwayWhenIdIsTaken() throws Exception {
    Model model = new ModelFullIndexed(null);

    Layer layer = new Layer();
    layer.setName(CreateHierarchyCommand.PATHWAY_LAYER_NAME);
    LayerRect layerRect = createLayerRect();
    layer.addLayerRect(layerRect);
    LayerRect layerRect2 = createLayerRect();
    layer.addLayerRect(layerRect2);
    model.addLayer(layer);

    Protein protein = createProtein();
    protein.setElementId("art0");
    model.addElement(protein);

    Protein protein2 = createProtein();
    protein2.setElementId("art1");
    model.addElement(protein2);

    CreateHierarchyCommand command = new CreateHierarchyCommand(model, 4, 80);
    command.createArtificials();

    PathwayCompartment pathway = (PathwayCompartment) model.getCompartments().get(0);

    assertNotNull(pathway);
  }

  @Test
  public void testProcessNotesForLayerText() throws Exception {

    Model model = new ModelFullIndexed(null);

    Layer layer = new Layer();
    layer.setName(CreateHierarchyCommand.PATHWAY_LAYER_NAME);
    LayerText layerText = new LayerText();
    layerText.setNotes("colors check\n"
        + "SemanticZoomLevelVisibility: 3\n"
        + "SemanticZoomLevelTransparency: 4\n"
        + "BackgroundColor:#0000ee\n"
        + "BorderColor:#00ee00");
    layer.addLayerText(layerText);
    model.addLayer(layer);

    Protein protein = createProtein();
    protein.setElementId("art0");
    model.addElement(protein);

    Protein protein2 = createProtein();
    protein2.setElementId("art1");
    model.addElement(protein2);

    CreateHierarchyCommand command = new CreateHierarchyCommand(model, 4, 80);
    command.createArtificials();

    PathwayCompartment pathway = (PathwayCompartment) model.getCompartments().get(0);

    assertNotNull(pathway);

    assertFalse(layerText.getNotes().contains("SemanticZoomLevelVisibility"));
  }

  @Test
  public void testCreatePathwayFromIgnoredLayer() throws Exception {
    Model model = new ModelFullIndexed(null);

    Layer layer = new Layer();
    layer.setName(CreateHierarchyCommand.PATHWAY_LAYER_NAME + "blah");
    LayerRect layerRect = createLayerRect();
    layer.addLayerRect(layerRect);
    model.addLayer(layer);

    CreateHierarchyCommand command = new CreateHierarchyCommand(model, 4, 80);
    command.createArtificials();

    assertEquals(0, model.getCompartments().size());
  }

  @Test
  public void deletePathwayLayer() {
    Model model = new ModelFullIndexed(null);

    Layer layer = new Layer();
    layer.setName(CreateHierarchyCommand.PATHWAY_LAYER_NAME);
    LayerRect layerRect = createLayerRect();
    layer.addLayerRect(layerRect);
    model.addLayer(layer);

    CreateHierarchyCommand command = new CreateHierarchyCommand(model, 4, 80);
    command.createArtificials();

    PathwayCompartment pathway = (PathwayCompartment) model.getCompartments().get(0);

    assertEquals(0, model.getLayers().size());
  }

  @Test
  public void testCreateImageFromText() {
    Model model = new ModelFullIndexed(null);

    Layer layer = new Layer();
    layer.setName(CreateHierarchyCommand.PATHWAY_LAYER_NAME + "blah");
    LayerText layerText = createLayerText();
    layer.addLayerText(layerText);
    model.addLayer(layer);

    CreateHierarchyCommand command = new CreateHierarchyCommand(model, 4, 80);
    command.createArtificials();

    assertEquals(0, model.getCompartments().size());
    assertEquals(1, layer.getImages().size());
    assertEquals(0, layer.getTexts().size());
  }

}
