package lcsb.mapviewer.model;

import lcsb.mapviewer.common.Comparator;
import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.comparator.SetComparator;
import lcsb.mapviewer.model.map.layout.graphics.Glyph;
import lcsb.mapviewer.model.map.layout.graphics.GlyphComparator;
import lcsb.mapviewer.model.map.model.ModelComparator;
import lcsb.mapviewer.model.map.model.ModelData;
import lcsb.mapviewer.model.map.model.ModelDataComparator;
import lcsb.mapviewer.model.overlay.DataOverlay;
import lcsb.mapviewer.model.overlay.DataOverlayComparator;
import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;

import java.util.HashSet;

public class ProjectComparator extends Comparator<Project> {

  private static final Logger logger = LogManager.getLogger();

  private final ModelComparator modelComparator;
  private final SetComparator<ModelData> modelDataSetComparator;
  private final SetComparator<DataOverlay> dataOverlaySetComparator;
  private final SetComparator<Glyph> glyphSetComparator;

  public ProjectComparator(final double epsilon) {
    super(Project.class);
    modelComparator = new ModelComparator(epsilon);
    modelDataSetComparator = new SetComparator<>(new ModelDataComparator(epsilon));
    glyphSetComparator = new SetComparator<>(new GlyphComparator());
    dataOverlaySetComparator = new SetComparator<>(new DataOverlayComparator(epsilon));
  }

  public ProjectComparator() {
    this(Configuration.EPSILON);
  }

  @Override
  protected int internalCompare(final Project arg0, final Project arg1) {
    int compareResult = glyphSetComparator.compare(new HashSet<>(arg0.getGlyphs()), new HashSet<>(arg1.getGlyphs()));
    if (compareResult != 0) {
      logger.debug("Glyphs different");
      return compareResult;
    }

    compareResult = modelComparator.compare(arg0.getTopModel(), arg1.getTopModel());
    if (compareResult != 0) {
      logger.debug("Top model different: {}, {}", arg0.getTopModel(), arg1.getTopModel());
      return compareResult;
    }

    if (modelDataSetComparator.compare(arg0.getModels(), arg1.getModels()) != 0) {
      logger.debug("Models different");
      return modelDataSetComparator.compare(arg0.getModels(), arg1.getModels());
    }

    int status = dataOverlaySetComparator.compare(new HashSet<>(arg0.getDataOverlays()), new HashSet<>(arg1.getDataOverlays()));
    if (status != 0) {
      logger.debug("Data overlays different");
      return status;
    }

    return 0;
  }

}
