package lcsb.mapviewer.model.overlay;

import lcsb.mapviewer.common.Comparator;
import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.comparator.SetComparator;
import lcsb.mapviewer.common.comparator.StringComparator;

import java.util.HashSet;

public class DataOverlayComparator extends Comparator<DataOverlay> {

  private final SetComparator<DataOverlayEntry> dataOverlayEntrySetComparator;

  public DataOverlayComparator(final double epsilon) {
    super(DataOverlay.class, true);
    this.dataOverlayEntrySetComparator = new SetComparator<>(new DataOverlayEntryComparator(epsilon));
  }

  public DataOverlayComparator() {
    this(Configuration.EPSILON);
  }

  @Override
  protected int internalCompare(final DataOverlay arg0, final DataOverlay arg1) {
    StringComparator stringComparator = new StringComparator();

    int status = stringComparator.compare(arg0.getName(), arg1.getName());
    if (status != 0) {
      logger.debug("Name different: {}, {}", arg0.getName(), arg1.getName());
      return status;
    }

    status = stringComparator.compare(arg0.getDescription(), arg1.getDescription());
    if (status != 0) {
      logger.debug("Description different: {}, {}", arg0.getDescription(), arg1.getDescription());
      return status;
    }

    status = stringComparator.compare(arg0.getGenomeVersion(), arg1.getGenomeVersion());
    if (status != 0) {
      logger.debug("GenomeVersion different: {}, {}", arg0.getGenomeVersion(), arg1.getGenomeVersion());
      return status;
    }

    status = stringComparator.compare(arg0.getGenomeVersion(), arg1.getGenomeVersion());
    if (status != 0) {
      logger.debug("GenomeVersion different: {}, {}", arg0.getGenomeVersion(), arg1.getGenomeVersion());
      return status;
    }

    status = dataOverlayEntrySetComparator.compare(new HashSet<>(arg0.getEntries()), new HashSet<>(arg1.getEntries()));
    if (status != 0) {
      logger.debug("Entries different");
      return status;
    }

    return 0;
  }
}
