package lcsb.mapviewer.model.overlay;

import lcsb.mapviewer.common.Comparator;
import lcsb.mapviewer.common.Configuration;
import lcsb.mapviewer.common.comparator.BooleanComparator;
import lcsb.mapviewer.common.comparator.ColorComparator;
import lcsb.mapviewer.common.comparator.DoubleComparator;
import lcsb.mapviewer.common.comparator.SetComparator;
import lcsb.mapviewer.common.comparator.StringComparator;
import lcsb.mapviewer.model.map.BioEntity;
import lcsb.mapviewer.model.map.MiriamData;
import lcsb.mapviewer.model.map.MiriamDataComparator;
import lcsb.mapviewer.modelutils.map.ClassNameComparator;

public class DataOverlayEntryComparator extends Comparator<DataOverlayEntry> {

  private final SetComparator<MiriamData> miriamDataSetComparator = new SetComparator<>(new MiriamDataComparator());

  private final SetComparator<String> stringSetComparator = new SetComparator<>(new StringComparator());
  private final SetComparator<Class<? extends BioEntity>> classSetComparator = new SetComparator<>(new ClassNameComparator<>());

  private final ColorComparator colorComparator = new ColorComparator();
  private final BooleanComparator booleanComparator = new BooleanComparator();
  private final DoubleComparator doubleComparator;

  public DataOverlayEntryComparator(final double epsilon) {
    super(DataOverlayEntry.class);
    doubleComparator = new DoubleComparator(epsilon);
  }

  public DataOverlayEntryComparator() {
    this(Configuration.EPSILON);
  }

  @Override
  protected int internalCompare(final DataOverlayEntry arg0, final DataOverlayEntry arg1) {
    StringComparator stringComparator = new StringComparator();

    int status = stringComparator.compare(arg0.getName(), arg1.getName());
    if (status != 0) {
      logger.debug("Name different: {}, {}", arg0.getName(), arg1.getName());
      return status;
    }

    status = stringComparator.compare(arg0.getDescription(), arg1.getDescription());
    if (status != 0) {
      logger.debug("Description different: {}, {}", arg0.getDescription(), arg1.getDescription());
      return status;
    }

    status = stringComparator.compare(arg0.getElementId(), arg1.getElementId());
    if (status != 0) {
      logger.debug("ElementId different: {}, {}", arg0.getElementId(), arg1.getElementId());
      return status;
    }

    status = stringComparator.compare(arg0.getModelName(), arg1.getModelName());
    if (status != 0) {
      logger.debug("ModelName different: {}, {}", arg0.getModelName(), arg1.getModelName());
      return status;
    }

    status = colorComparator.compare(arg0.getColor(), arg1.getColor());
    if (status != 0) {
      logger.debug("Color different: {}, {}", arg0.getColor(), arg1.getColor());
      return status;
    }

    status = miriamDataSetComparator.compare(arg0.getMiriamData(), arg1.getMiriamData());
    if (status != 0) {
      logger.debug("MiriamData different: {}, {}", arg0.getMiriamData(), arg1.getMiriamData());
      return status;
    }

    status = stringSetComparator.compare(arg0.getCompartments(), arg1.getCompartments());
    if (status != 0) {
      logger.debug("Compartments different: {}, {}", arg0.getCompartments(), arg1.getCompartments());
      return status;
    }

    status = doubleComparator.compare(arg0.getLineWidth(), arg1.getLineWidth());
    if (status != 0) {
      logger.debug("LineWidth different: {}, {}", arg0.getLineWidth(), arg1.getLineWidth());
      return status;
    }

    status = booleanComparator.compare(arg0.getReverseReaction(), arg1.getReverseReaction());
    if (status != 0) {
      logger.debug("ReverseReaction different: {}, {}", arg0.getReverseReaction(), arg1.getReverseReaction());
      return status;
    }

    status = doubleComparator.compare(arg0.getValue(), arg1.getValue());
    if (status != 0) {
      logger.debug("Value different: {}, {}", arg0.getValue(), arg1.getValue());
      return status;
    }

    status = classSetComparator.compare(arg0.getTypes(), arg1.getTypes());
    if (status != 0) {
      logger.debug("Types different: {}, {}", arg0.getTypes(), arg1.getTypes());
      return status;
    }

    return 0;
  }
}
